@@ -17,6 +17,8 @@ from .views import (
 from .schemas import JSONFieldException, URLParamValidationError, InvalidSession
 
 from taler.util.amount import CurrencyMismatchError, AmountFormatError
+from taler.util.taler_error_codes import ErrorCode
+from http import HTTPStatus
 
 LOGGER = logging.getLogger()
 
@@ -95,18 +97,25 @@ class ExceptionMiddleware:
         if so.  Otherwise, it lets the native handler operate.
         """
         LOGGER.error(f"Error: {exception}, while serving {request.get_full_path()}")
-        LOGGER.error(f"Stack trace: {traceback.format_exc()}")
+        # LOGGER.error(f"Stack trace: {traceback.format_exc()}")
 
         if hasattr(exception, "taler_error_code"):
             render_to = self.render.get(request.path)
-
             if not render_to:
-                return JsonResponse(
-                    {"code": exception.taler_error_code, "error": exception.hint},
+                response = JsonResponse(
+                    dict(code=exception.taler_error_code.value, error=exception.hint),
                     status=exception.http_status_code,
                 )
-            set_session_hint(request, failure=True, success=False, hint=exception.hint)
+                response["Access-Control-Allow-Origin"] = "*"
+                return response
+            set_session_hint(request, success=False, hint=exception.hint)
             return redirect(render_to)
         else:
-            return JsonResponse(dict(code=5300, hint="unexpected exception", exception=str(exception)), status=500)
-
+            return JsonResponse(
+                dict(
+                    code=ErrorCode.BANK_UNMANAGED_EXCEPTION,
+                    hint="unexpected exception",
+                    exception=str(exception),
+                ),
+                status=HTTPStatus.INTERNAL_SERVER_ERROR,
+            )
