@@ -19,6 +19,7 @@
 
 import json
 import time
+import uuid
 import zlib
 import timeit
 import logging
@@ -31,7 +32,7 @@ from django.urls import reverse
 from django.conf import settings
 from django.contrib.auth.models import User
 from mock import patch, MagicMock
-from .models import BankAccount, BankTransaction
+from .models import BankAccount, BankTransaction, TalerWithdrawOperation
 from . import urls
 from .views import wire_transfer, get_reserve_pub, PaytoParse
 from taler.util.amount import (
@@ -52,6 +53,7 @@ fileHandler.setFormatter(
 )
 LOGGER.addHandler(fileHandler)
 
+
 def make_auth_line(username, password):
     credentials = "%s:%s" % (username, password)
     b64enc = base64.b64encode(bytes(credentials, "utf-8"))
@@ -67,6 +69,7 @@ def clear_db():
         cursor.execute("ALTER SEQUENCE app_bankaccount_account_no_seq" " RESTART")
         cursor.execute("ALTER SEQUENCE app_banktransaction_id_seq RESTART")
 
+
 class WireGatewayTestCase(TestCase):
     def setUp(self):
         clear_db()
@@ -76,12 +79,12 @@ class WireGatewayTestCase(TestCase):
         customer.save()
         exchange_bank_account = BankAccount(
             user=exchange,
-            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0))
+            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0)),
         )
         exchange_bank_account.save()
         customer_bank_account = BankAccount(
             user=customer,
-            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0))
+            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0)),
         )
         customer_bank_account.save()
         self.client = Client()
@@ -91,9 +94,12 @@ class WireGatewayTestCase(TestCase):
             reverse("twg-add-incoming", kwargs=dict(acct_id="RandomExchange")),
             HTTP_AUTHORIZATION=make_auth_line("RandomExchange", "XYZ"),
             content_type="application/json",
-            data=dict(amount=f"{settings.TALER_CURRENCY}:10",
-                      reserve_pub="FXWC2JHBY8B0XE2MMGAJ9TGPY307TN12HVEKYSTN6HE3GTHTF8XG",
-                      debit_account="payto://x-taler-bank/localhost/RandomCustomer"))
+            data=dict(
+                amount=f"{settings.TALER_CURRENCY}:10",
+                reserve_pub="FXWC2JHBY8B0XE2MMGAJ9TGPY307TN12HVEKYSTN6HE3GTHTF8XG",
+                debit_account="payto://x-taler-bank/localhost/RandomCustomer",
+            ),
+        )
         self.assertEqual(r.status_code, 200)
 
         # Test incoming transfers of Exchange.
@@ -110,8 +116,12 @@ class WireGatewayTestCase(TestCase):
             HTTP_AUTHORIZATION=make_auth_line("RandomExchange", "XYZ"),
             content_type="application/json",
             data=dict(
-                request_uid="0", amount=f"{settings.TALER_CURRENCY}:3",
-                exchange_base_url="mock", wtid="123", credit_account="payto://x-taler-bank/localhost/RandomCustomer")
+                request_uid="0",
+                amount=f"{settings.TALER_CURRENCY}:3",
+                exchange_base_url="mock",
+                wtid="123",
+                credit_account="payto://x-taler-bank/localhost/RandomCustomer",
+            ),
         )
         r = self.client.get(
             reverse("twg-history-outgoing", kwargs=dict(acct_id="RandomExchange")),
@@ -120,22 +130,62 @@ class WireGatewayTestCase(TestCase):
         )
         self.assertEqual(r.status_code, 200)
 
-# Testing the "Integration API":
-#      has three operations: 1) /config, 2,3) create, see status of withdrawal.
-class IntegrationConfigTestCase(TestCase):
+
+class IntegrationApiTestCase(TestCase):
+    def setUp(self):
+        clear_db()
+
+        self.exchange = User.objects.create_user(
+            username="RandomExchange", password="XYZ"
+        )
+        self.exchange.save()
+        self.exchange_bank_account = BankAccount(
+            user=self.exchange,
+            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0)),
+        )
+        self.exchange_bank_account.save()
+
+        self.user = User.objects.create_user(username="RandomUser", password="XYZ")
+        self.user.save()
+        self.user_bank_account = BankAccount(
+            user=self.user,
+            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0)),
+        )
+        self.user_bank_account.save()
+        self.client = Client()
+
     def test_config(self):
         c = Client()
         r = c.get("/config")
         self.assertEqual(r.status_code, 200)
 
-class AccessAndIntegrationApiWithdrawTestCase(TestCase):
+    def test_withdraw(self):
+        operation = TalerWithdrawOperation(
+            amount=Amount(settings.TALER_CURRENCY, 100, 0),
+            withdraw_account=self.user_bank_account,
+        )
+        operation.save()
+        r = self.client.post(
+            reverse(
+                "api-withdraw-operation", kwargs=dict(withdraw_id=operation.withdraw_id)
+            ),
+            data=dict(
+                reserve_pub="reserve-public-key",
+                selected_exchange=f"payto://x-taler-bank/localhost/RandomExchange",
+            ),
+            content_type="application/json",
+        )
+        self.assertEqual(r.status_code, 200)
+
+
+class AccessApiWithdrawTestCase(TestCase):
     def setUp(self):
         clear_db()
         self.user = User.objects.create_user(username="RandomUser", password="XYZ")
         self.user.save()
         self.user_bank_account = BankAccount(
             user=self.user,
-            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0))
+            balance=SignedAmount(True, Amount(settings.TALER_CURRENCY, 100, 0)),
         )
         self.user_bank_account.save()
         self.client = Client()
@@ -145,7 +195,7 @@ class AccessAndIntegrationApiWithdrawTestCase(TestCase):
             reverse("access-api-withdrawal", kwargs=dict(acct_id="RandomUser")),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
             content_type="application/json",
-            data=dict(amount=f"{settings.TALER_CURRENCY}:5")
+            data=dict(amount=f"{settings.TALER_CURRENCY}:5"),
         )
         self.assertEqual(r.status_code, 200)
         data = r.content.decode("utf-8")
@@ -160,8 +210,10 @@ class AccessAndIntegrationApiWithdrawTestCase(TestCase):
     def test_accees_withdraw_status(self):
         withdrawal_id = self.create_withdrawal()
         r = self.client.get(
-            reverse("access-api-withdrawal-status",
-                    kwargs=dict(acct_id="RandomUser", wid=withdrawal_id)),
+            reverse(
+                "access-api-withdrawal-status",
+                kwargs=dict(acct_id="RandomUser", wid=withdrawal_id),
+            ),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 200)
@@ -169,8 +221,10 @@ class AccessAndIntegrationApiWithdrawTestCase(TestCase):
     def test_accees_withdraw_abort(self):
         withdrawal_id = self.create_withdrawal()
         r = self.client.post(
-            reverse("access-api-withdrawal-abort",
-                    kwargs=dict(acct_id="RandomUser", wid=withdrawal_id)),
+            reverse(
+                "access-api-withdrawal-abort",
+                kwargs=dict(acct_id="RandomUser", wid=withdrawal_id),
+            ),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 200)
@@ -178,8 +232,10 @@ class AccessAndIntegrationApiWithdrawTestCase(TestCase):
     def test_accees_withdraw_confirm(self):
         withdrawal_id = self.create_withdrawal()
         r = self.client.post(
-            reverse("access-api-withdrawal-confirm",
-                    kwargs=dict(acct_id="RandomUser", wid=withdrawal_id)),
+            reverse(
+                "access-api-withdrawal-confirm",
+                kwargs=dict(acct_id="RandomUser", wid=withdrawal_id),
+            ),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 200)
@@ -187,34 +243,50 @@ class AccessAndIntegrationApiWithdrawTestCase(TestCase):
     def test_accees_withdraw_abort_then_confirm(self):
         withdrawal_id = self.create_withdrawal()
         r = self.client.post(
-            reverse("access-api-withdrawal-abort",
-                    kwargs=dict(acct_id="RandomUser", wid=withdrawal_id)),
+            reverse(
+                "access-api-withdrawal-abort",
+                kwargs=dict(acct_id="RandomUser", wid=withdrawal_id),
+            ),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 200)
         r = self.client.post(
-            reverse("access-api-withdrawal-confirm",
-                    kwargs=dict(acct_id="RandomUser", wid=withdrawal_id)),
+            reverse(
+                "access-api-withdrawal-confirm",
+                kwargs=dict(acct_id="RandomUser", wid=withdrawal_id),
+            ),
             HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 409)
 
     def test_integration_api_withdraw_status(self):
         wid = self.create_withdrawal()
-        r = self.client.get(reverse("api-withdraw-operation", kwargs=dict(withdraw_id=wid)))
+        r = self.client.get(
+            reverse(
+                "access-api-withdrawal-status",
+                kwargs=dict(acct_id="RandomUser", wid=wid),
+            ),
+            HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
+        )
         self.assertEqual(r.status_code, 200)
 
     def test_integration_api_withdraw_confirm(self):
         wid = self.create_withdrawal()
         r = self.client.post(
-            reverse("api-withdraw-operation", kwargs=dict(withdraw_id=wid)),
+            reverse(
+                "access-api-withdrawal-confirm",
+                kwargs=dict(acct_id="RandomUser", wid=wid),
+            ),
             data=dict(
                 reserve_pub="FXWC2JHBY8B0XE2MMGAJ9TGPY307TN12HVEKYSTN6HE3GTHTF8XG",
-                selected_exchange="payto://x-taler-bank/localhost/RandomUser"),
-            content_type="application/json"
+                selected_exchange="payto://x-taler-bank/localhost/RandomUser",
+            ),
+            content_type="application/json",
+            HTTP_AUTHORIZATION=make_auth_line("RandomUser", "XYZ"),
         )
         self.assertEqual(r.status_code, 200)
 
+
 class AccessApiBalanceTestCase(TestCase):
     def setUp(self):
         clear_db()
@@ -231,6 +303,7 @@ class AccessApiBalanceTestCase(TestCase):
         )
         self.assertEqual(r.status_code, 200)
 
+
 class AccessApiTestingRegistrationTestCase(TestCase):
     def setUp(self):
         clear_db()
@@ -241,22 +314,40 @@ class AccessApiTestingRegistrationTestCase(TestCase):
 
     def test_testing_registration(self):
         c = Client()
-        r = c.post(reverse("testing-withdraw-register"),
-                   data=dict(username="x", password="y"),
-                   content_type="application/json")
+        r = c.post(
+            reverse("testing-withdraw-register"),
+            data=dict(username="x", password="y"),
+            content_type="application/json",
+        )
         self.assertEqual(r.status_code, 200)
 
+
 class ReservePubExtractionTestCase(TestCase):
     def test_extraction(self):
-        self.assertTrue(get_reserve_pub("0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG"))
-        self.assertTrue(get_reserve_pub("0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG other data"))
+        self.assertTrue(
+            get_reserve_pub("0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG")
+        )
+        self.assertTrue(
+            get_reserve_pub(
+                "0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG other data"
+            )
+        )
         self.assertFalse(get_reserve_pub("not a reserve public key"))
 
+
 class PaytoParseTestCase(TestCase):
     def test_payto_parse(self):
-        parsed = PaytoParse("payto://x-taler-bank/bank.int.taler.net/Exchange?subject=0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG")
+        parsed = PaytoParse(
+            "payto://x-taler-bank/bank.int.taler.net/Exchange?subject=0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG&amount=EUR:1"
+        )
         self.assertTrue("Exchange" == parsed.account)
-        self.assertTrue("0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG" == parsed.subject)
+        self.assertTrue(
+            "0T096A11M57GWGG0P6ZM9Z8G5829BFJFH2AN9R5T80FJ931DX7GG" == parsed.subject
+        )
+        self.assertTrue(parsed.amount.value == 1)
+        self.assertTrue(parsed.amount.fraction == 0)
+        self.assertTrue(parsed.amount.currency == "EUR")
+
 
 class PublicAccountsTestCase(TestCase):
     def setUp(self):
@@ -335,36 +426,6 @@ class WithdrawTestCase(TestCase):
         clear_db()
 
 
-class InternalWireTransferTestCase(TestCase):
-    def setUp(self):
-        BankAccount(
-            user=User.objects.create_user(username="give_money", password="gm")
-        ).save()
-        self.take_money = BankAccount(
-            user=User.objects.create_user(username="take_money"), account_no=4
-        )
-        self.take_money.save()
-
-    def tearDown(self):
-        clear_db()
-
-    def test_internal_wire_transfer(self):
-        client = Client()
-        client.login(username="give_money", password="gm")
-        response = client.post(
-            reverse("profile", urlconf=urls),
-            {
-                "amount": 3.0,
-                "receiver": self.take_money.account_no,
-                "subject": "charity",
-            },
-        )
-        take_money = BankAccount.objects.get(account_no=4)
-        r = SignedAmount.parse(f"{settings.TALER_CURRENCY}:3.0")
-        self.assertEqual(take_money.balance, r)
-        self.assertEqual(302, response.status_code)
-
-
 class RegisterTestCase(TestCase):
     """User registration"""
 
@@ -383,7 +444,7 @@ class RegisterTestCase(TestCase):
             {"username": "test_register", "password": "test_register"},
             follow=True,
         )
-        self.assertIn(("/profile", 302), response.redirect_chain)
+        self.assertIn(("/en/profile", 302), response.redirect_chain)
         # this assertion tests "/profile""s view
         self.assertEqual(200, response.status_code)
 
@@ -454,6 +515,7 @@ class LoginTestCase(TestCase):
         data = response.content.decode("utf-8")
         self.assertEqual(401, response.status_code)
 
+
 class AddIncomingTestCase(TestCase):
     """Test money transfer's API"""
 
