@@ -1,6 +1,6 @@
 ##
 # This file is part of TALER
-# (C) 2014, 2015, 2016 Taler Systems SA
+# (C) 2014, 2015, 2016, 2020 Taler Systems SA
 #
 # TALER is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as
@@ -38,6 +38,7 @@ from django.conf import settings
 from django.contrib.auth.decorators import login_required
 from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.http import require_POST, require_GET
+from django.utils.translation import gettext
 from django.views.decorators.http import require_http_methods
 from django.urls import reverse, get_script_prefix
 from django.contrib.auth.models import User
@@ -48,6 +49,9 @@ from django.core.exceptions import ObjectDoesNotExist
 from datetime import datetime
 from .models import BankAccount, BankTransaction, TalerWithdrawOperation
 from taler.util.amount import Amount, SignedAmount
+from taler.util.taler_error_codes import ErrorCode
+from http import HTTPStatus
+
 import qrcode
 import qrcode.image.svg
 import lxml
@@ -69,6 +73,32 @@ LOGGER = logging.getLogger(__name__)
 # can handle (because of the wallet).
 UINT64_MAX = (2 ** 64) - 1
 
+##
+# Decorator function that authenticates requests by fetching
+# the credentials over the HTTP requests headers.
+#
+# @param view_func function that will be called after the
+#        authentication, and that will usually serve the requested
+#        endpoint.
+# @return FIXME.
+def login_via_headers(view_func):
+    def _decorator(request, *args, **kwargs):
+        user_account = basic_auth(request)
+        if not user_account:
+            raise LoginFailed("authentication failed")
+        return view_func(request, user_account, *args, **kwargs)
+
+    return wraps(view_func)(_decorator)
+
+
+def allow_origin_star(view_func):
+    def _decorator(request, *args, **kwargs):
+        response = view_func(request, *args, **kwargs)
+        response["Access-Control-Allow-Origin"] = "*"
+        return response
+
+    return wraps(view_func)(_decorator)
+
 
 class PaytoParse:
     def __init__(self, payto_uri):
@@ -81,6 +111,7 @@ class PaytoParse:
         self.account = path_as_list[-1]
         params = dict(parse_qsl(parsed_payto.query))
         self.subject = params.get("subject")
+        self.amount = Amount.parse(params.get("amount"))
 
 
 ##
@@ -90,24 +121,24 @@ class LoginFailed(Exception):
     def __init__(self, msg):
         super(LoginFailed, self).__init__(msg)
         self.hint = "Wrong password given"
-        self.http_status_code = 401
-        self.taler_error_code = 5109
+        self.http_status_code = HTTPStatus.UNAUTHORIZED
+        self.taler_error_code = ErrorCode.BANK_LOGIN_FAILED
 
 
 class InvalidInputData(Exception):
     def __init__(self, msg):
         super(InvalidInputData, self).__init__(msg)
         self.hint = msg  # should mention the picked username
-        self.http_status_code = 400
-        self.taler_error_code = 5400
+        self.http_status_code = HTTPStatus.BAD_REQUEST
+        self.taler_error_code = ErrorCode.BANK_SOFT_EXCEPTION
 
 
 class UsernameUnavailable(Exception):
     def __init__(self, msg):
         super(UsernameUnavailable, self).__init__(msg)
         self.hint = msg  # should mention the picked username
-        self.http_status_code = 406
-        self.taler_error_code = 5400
+        self.http_status_code = HTTPStatus.NOT_ACCEPTABLE
+        self.taler_error_code = ErrorCode.BANK_SOFT_EXCEPTION
 
 
 ##
@@ -117,7 +148,7 @@ class PrivateAccountException(Exception):
     def __init__(self, msg):
         super(PrivateAccountException, self).__init__(msg)
         self.hint = "Cannot show history from private persons accounts"
-        self.http_status_code = 402
+        self.http_status_code = HTTPStatus.FORBIDDEN
 
 
 ##
@@ -127,8 +158,8 @@ class DebitLimitException(Exception):
     def __init__(self, msg):
         super(DebitLimitException, self).__init__(msg)
         self.hint = "Payment aborted for insufficient credit"
-        self.http_status_code = 406
-        self.taler_error_code = 5103
+        self.http_status_code = HTTPStatus.FORBIDDEN
+        self.taler_error_code = ErrorCode.BANK_UNALLOWED_DEBIT
 
 
 ##
@@ -139,16 +170,16 @@ class SameAccountException(Exception):
     def __init__(self, msg):
         super(SameAccountException, self).__init__(msg)
         self.hint = "Cannot send payment to oneself."
-        self.http_status_code = 403
-        self.taler_error_code = 5102
+        self.http_status_code = HTTPStatus.BAD_REQUEST
+        self.taler_error_code = ErrorCode.BANK_SAME_ACCOUNT
 
 
 class UnhandledException(Exception):
     def __init__(self, msg="Unhandled exception happened!"):
         super(UnhandledException, self).__init__(msg)
         self.hint = msg
-        self.http_status_code = 500
-        self.taler_error_code = 5300
+        self.http_status_code = HTTPStatus.INTERNAL_SERVER_ERROR
+        self.taler_error_code = ErrorCode.BANK_UNMANAGED_EXCEPTION
 
 
 ##
@@ -204,13 +235,14 @@ def get_session_flag(request, name):
 def get_session_hint(request):
     ret = True, ""
     if "hint" in request.session:
-        ret = request.session["hint"] 
+        ret = request.session["hint"]
         del request.session["hint"]
     return ret
 
+
 def set_session_hint(request, success, hint):
     if "hint" in request.session:
-        LOGGER.warning(f"Overriding a non consumed hint")
+        LOGGER.warning("Overriding a non consumed hint")
         del request.session["hint"]
     request.session["hint"] = success, hint
 
@@ -272,19 +304,16 @@ class InputDatalist(forms.TextInput):
         return html + datalist
 
 
-class WTForm(forms.Form):
-    """
-    Form for sending wire transfers.  It usually appears in the
-    user profile page.
-    """
+class PaytoTransferForm(forms.Form):
+    address = forms.CharField()
 
-    amount = forms.FloatField(
-        min_value=0.1, widget=forms.NumberInput(attrs={"class": "currency-input"})
+
+@login_required
+def payto_form(request):
+    context = dict(
+        currency=request.user.bankaccount.balance.amount.currency,
     )
-    receiver = forms.IntegerField(
-        min_value=1, widget=InputDatalist(predefined_accounts_list, "receiver")
-    )
-    subject = forms.CharField()
+    return render(request, "payto_wiretransfer.html", context)
 
 
 ##
@@ -303,28 +332,6 @@ class WTForm(forms.Form):
 # @return Django-specific HTTP response object.
 @login_required
 def profile_page(request):
-    if request.method == "POST":
-        # WTForm ~ Wire Transfer Form.
-        wtf = WTForm(request.POST)
-        if wtf.is_valid():
-            amount_parts = (
-                settings.TALER_CURRENCY,
-                wtf.cleaned_data.get("amount") + 0.0,
-            )
-            wire_transfer(
-                Amount.parse("%s:%s" % amount_parts),
-                BankAccount.objects.get(user=request.user),
-                BankAccount.objects.get(account_no=wtf.cleaned_data.get("receiver")),
-                wtf.cleaned_data.get("subject"),
-            )
-
-            set_session_hint(
-                request, success=True, hint="Wire transfer successful!"
-            )
-
-            return redirect("profile")
-
-    wtf = WTForm()
     is_success, hint = get_session_hint(request)
     context = dict(
         name=request.user.username,
@@ -334,7 +341,6 @@ def profile_page(request):
         precision=settings.TALER_DIGITS,
         currency=request.user.bankaccount.balance.amount.currency,
         account_no=request.user.bankaccount.account_no,
-        wt_form=wtf,
         history=extract_history(request.user.bankaccount, -1 * (UINT64_MAX / 2 / 2)),
     )
     if settings.TALER_SUGGESTED_EXCHANGE:
@@ -344,20 +350,39 @@ def profile_page(request):
     if "just_withdrawn" in request.session:
         del request.session["just_withdrawn"]
         response["Taler"] = "taler://notify-reserve/"
-        response.status_code = 202
+        response.status_code = HTTPStatus.ACCEPTED
     return response
 
 
+@login_required
+@require_POST
+def payto_transfer(request):
+    data = PaytoTransferForm(request.POST)
+    if not data.is_valid():
+        set_session_hint(request, success=False, hint=gettext("Bad form submitted!"))
+        return redirect("profile")
+
+    parsed_address = PaytoParse(data.cleaned_data.get("address"))
+    wire_transfer(
+        parsed_address.amount,
+        BankAccount.objects.get(user=request.user),
+        BankAccount.objects.get(account_no=parsed_address.account),
+        parsed_address.subject,
+    )
+    set_session_hint(request, success=True, hint=gettext("Wire transfer successful!"))
+    return redirect("profile")
+
+
 ##
 # Helper function that hashes its input.  Usually
 # used to hash the response to the math CAPTCHA.
 #
-# @param ans the plain text answer to hash.
-# @return the hashed version of @a ans.
-def hash_answer(ans):
+# @param answer the plain text answer to hash.
+# @return the hashed version of @a answer.
+def hash_answer(answer):
     hasher = hashlib.new("sha1")
     hasher.update(settings.SECRET_KEY.encode("utf-8"))
-    hasher.update(ans.encode("utf-8"))
+    hasher.update(answer.encode("utf-8"))
     return hasher.hexdigest()
 
 
@@ -441,17 +466,21 @@ def register_headless(request):
     registering a user.
     """
     if not settings.ALLOW_REGISTRATIONS:
-        return JsonResponse(dict(error="registrations are not allowed"), status=403)
+        return JsonResponse(
+            dict(error="registrations are not allowed"), status=HTTPStatus.FORBIDDEN
+        )
     username = expect_json_body_str(request, "username")
     password = expect_json_body_str(request, "password")
     try:
         internal_register(username, password)
     except UsernameUnavailable:
-        return JsonResponse(dict(hint="username unavailable"), status=409)  # Conflict
+        return JsonResponse(
+            dict(hint="username unavailable"), status=HTTPStatus.CONFLICT
+        )
     except InvalidInputData:
-        return HttpResponse(status=406)  # Not Acceptable
+        return HttpResponse(status=HTTPStatus.NOT_ACCEPTABLE)  # WTF? BAD REQUEST?
 
-    return HttpResponse(status=200)
+    return HttpResponse(status=HTTPStatus.OK)
 
 
 def register(request):
@@ -463,7 +492,9 @@ def register(request):
     """
     if not settings.ALLOW_REGISTRATIONS:
         # FIXME: shouldn't be JSON!
-        return JsonResponse(dict(error="registrations are not allowed"), status=403)
+        return JsonResponse(
+            dict(error="registrations are not allowed"), status=HTTPStatus.FORBIDDEN
+        )
     if request.method != "POST":
         return render(request, "register.html")
 
@@ -496,9 +527,7 @@ def register(request):
             {"wrong": True, "hint": "Out of business, cannot admit new customers."},
         )
 
-    set_session_hint(
-        request, success=True, hint="Registration successful!"
-    )
+    set_session_hint(request, success=True, hint=gettext("Registration successful!"))
 
     django.contrib.auth.login(request, user)
     return redirect("profile")
@@ -523,11 +552,12 @@ def config_view(request):
         dict(
             version="0:0:0", currency=settings.TALER_CURRENCY, name="taler-bank-access"
         ),
-        status=200,
+        status=HTTPStatus.OK,
     )
 
 
 @require_GET
+@allow_origin_star
 def api_config(request):
     """
     Config query of the taler bank integration api
@@ -538,7 +568,7 @@ def api_config(request):
             currency=settings.TALER_CURRENCY,
             name="taler-bank-integration",
         ),
-        status=200,
+        status=HTTPStatus.OK,
     )
 
 
@@ -612,36 +642,24 @@ def serve_public_accounts(request, name=None, page=None):
 
     pages = list(range(1, num_pages + 1))
 
+    is_success, hint = get_session_hint(request)
     context = dict(
+        is_success=is_success,
+        hint=hint,
         current_page=page,
         back=page - 1 if page > 1 else None,
         forth=page + 1 if page < num_pages else None,
         public_accounts=public_accounts,
         selected_account=dict(
-            name=name, number=user.bankaccount.account_no, history=history,
+            name=name,
+            number=user.bankaccount.account_no,
+            history=history,
         ),
         pages=pages,
     )
     return render(request, "public_accounts.html", context)
 
 
-##
-# Decorator function that authenticates requests by fetching
-# the credentials over the HTTP requests headers.
-#
-# @param view_func function that will be called after the
-#        authentication, and that will usually serve the requested
-#        endpoint.
-# @return FIXME.
-def login_via_headers(view_func):
-    def _decorator(request, *args, **kwargs):
-        user_account = basic_auth(request)
-        if not user_account:
-            raise LoginFailed("authentication failed")
-        return view_func(request, user_account, *args, **kwargs)
-    return wraps(view_func)(_decorator)
-
-
 ##
 # Build the DB query switch based on the "direction" history
 # argument given by the user.
@@ -757,7 +775,7 @@ def serve_history(request, user_account):
 
     history = build_history_response(qs, args.get("cancelled", "show"), user_account)
 
-    return JsonResponse(dict(data=history), status=200)
+    return JsonResponse(dict(data=history), status=HTTPStatus.OK)
 
 
 def expect_json_body_str(request, param_name):
@@ -801,7 +819,7 @@ def twg_base(request, acct_id):
     check if the account is up, should not normally be used
     for anything else.
     """
-    return JsonResponse(dict(), status=200)
+    return JsonResponse(dict(), status=HTTPStatus.OK)
 
 
 @require_GET
@@ -817,7 +835,7 @@ def twg_config(request, acct_id):
             name="taler-wire-gateway",
             currency=settings.TALER_CURRENCY,
         ),
-        status=200,
+        status=HTTPStatus.OK,
     )
 
 
@@ -849,7 +867,12 @@ def twg_add_incoming(request, user_account, acct_id):
     debit_account = BankAccount.objects.get(user=debit_user)
     subject = f"{reserve_pub}"
 
-    wtrans = wire_transfer(amount, debit_account, exchange_account, subject,)
+    wtrans = wire_transfer(
+        amount,
+        debit_account,
+        exchange_account,
+        subject,
+    )
 
     return JsonResponse(
         {
@@ -888,7 +911,11 @@ def twg_transfer(request, user_account, acct_id):
         LOGGER.error(f"credit account '{credit_account_name}' does not exist")
         # FIXME: use EC from taler-util library
         return JsonResponse(
-            dict(code=5110, error="credit account does not exist"), status=404
+            dict(
+                code=ErrorCode.BANK_UNKNOWN_ACCOUNT,
+                error="credit account does not exist",
+            ),
+            status=404,
         )
     credit_account = BankAccount.objects.get(user=credit_user)
 
@@ -936,7 +963,12 @@ def twg_history_incoming(request, user_account, acct_id):
         start = None
     else:
         start = int(start_str)
-    qs = query_history(user_account.bankaccount, "credit", delta, start,)
+    qs = query_history(
+        user_account.bankaccount,
+        "credit",
+        delta,
+        start,
+    )
     for item in qs:
         rp = get_reserve_pub(item.subject)
         if rp is None:
@@ -951,7 +983,7 @@ def twg_history_incoming(request, user_account, acct_id):
                 debit_account=get_payto_from_account(request, item.debit_account),
             )
         )
-    return JsonResponse(dict(incoming_transactions=history), status=200)
+    return JsonResponse(dict(incoming_transactions=history), status=HTTPStatus.OK)
 
 
 @require_GET
@@ -964,7 +996,12 @@ def twg_history_outgoing(request, user_account, acct_id):
         start = None
     else:
         start = int(start_str)
-    qs = query_history(user_account.bankaccount, "debit", delta, start,)
+    qs = query_history(
+        user_account.bankaccount,
+        "debit",
+        delta,
+        start,
+    )
     for item in qs:
         # FIXME: proper parsing, more structure in subject
         wtid, exchange_base_url = item.subject.split(" ")
@@ -979,7 +1016,7 @@ def twg_history_outgoing(request, user_account, acct_id):
                 debit_account=get_payto_from_account(request, item.debit_account),
             )
         )
-    return JsonResponse(dict(outgoing_transactions=history), status=200)
+    return JsonResponse(dict(outgoing_transactions=history), status=HTTPStatus.OK)
 
 
 ##
@@ -1034,13 +1071,16 @@ def withdraw_headless(request, user):
 
     exchange_payto = data.get("exchange_payto_uri")
     if not exchange_payto:
-        return JsonResponse(dict(hint="exchange_payto_uri missig"), status=400)
+        return JsonResponse(
+            dict(hint="exchange_payto_uri missig"), status=HTTPStatus.BAD_REQUEST
+        )
     exchange_account_name = get_acct_from_payto(exchange_payto)
     try:
         exchange_user = User.objects.get(username=exchange_account_name)
     except User.DoesNotExist:
         return JsonResponse(
-            dict(hint="exchange bank account does not exist"), status=404
+            dict(hint="exchange bank account does not exist"),
+            status=HTTPStatus.NOT_FOUND,
         )
     exchange_bankaccount = exchange_user.bankaccount
     wire_transfer(
@@ -1054,6 +1094,7 @@ def withdraw_headless(request, user):
 
 
 @csrf_exempt
+@allow_origin_star
 def api_withdraw_operation(request, withdraw_id):
     """
     Endpoint used by the browser and wallet to check withdraw status and
@@ -1062,7 +1103,9 @@ def api_withdraw_operation(request, withdraw_id):
     try:
         op = TalerWithdrawOperation.objects.get(withdraw_id=withdraw_id)
     except ObjectDoesNotExist:
-        return JsonResponse(dict(error="withdraw operation does not exist"), status=404)
+        return JsonResponse(
+            dict(error="withdraw operation does not exist"), status=HTTPStatus.NOT_FOUND
+        )
 
     if request.method == "POST":
         data = json.loads(decode_body(request))
@@ -1070,17 +1113,27 @@ def api_withdraw_operation(request, withdraw_id):
         try:
             exchange_account_name = get_acct_from_payto(exchange_payto_uri)
         except:
-            return JsonResponse(dict(error="exchange payto URI malformed"), status=400)
+            return JsonResponse(
+                dict(error="exchange payto URI malformed"),
+                status=HTTPStatus.BAD_REQUEST,
+            )
         try:
             exchange_user = User.objects.get(username=exchange_account_name)
         except User.DoesNotExist:
             return JsonResponse(
-                dict(code=5110, hint="bank account in payto URI unknown"), status=400
+                dict(
+                    code=ErrorCode.BANK_UNKNOWN_ACCOUNT,
+                    hint="bank account in payto URI unknown",
+                ),
+                status=HTTPStatus.BAD_REQUEST,  # WTF? NOT_FOUND?
             )
         exchange_account = exchange_user.bankaccount
         selected_reserve_pub = data.get("reserve_pub")
         if not isinstance(selected_reserve_pub, str):
-            return JsonResponse(dict(error="reserve_pub must be a string"), status=400)
+            return JsonResponse(
+                dict(error="reserve_pub must be a string"),
+                status=HTTPStatus.BAD_REQUEST,
+            )
         if op.selection_done:
             if (
                 op.selected_exchange_account != exchange_account
@@ -1088,9 +1141,10 @@ def api_withdraw_operation(request, withdraw_id):
             ):
                 return JsonResponse(
                     dict(
-                        code=5600, hint="selection of withdraw parameters already done"
+                        code=ErrorCode.BANK_WITHDRAWAL_OPERATION_RESERVE_SELECTION_CONFLICT,
+                        hint="selection of withdraw parameters already done",
                     ),
-                    status=409,
+                    status=HTTPStatus.CONFLICT,
                 )
         else:
             with transaction.atomic():
@@ -1131,7 +1185,10 @@ def api_withdraw_operation(request, withdraw_id):
             )
         )
     else:
-        return JsonResponse(dict(error="only GET and POST are allowed"), status=305)
+        return JsonResponse(
+            dict(error="only GET and POST are allowed"),
+            status=HTTPStatus.METHOD_NOT_ALLOWED,
+        )
 
 
 @login_required
@@ -1202,7 +1259,9 @@ def confirm_withdrawal(request, withdraw_id):
                 type(hashed_attempt),
                 type(request.POST.get("pin_1")),
             )
-            set_session_hint(request, success=False, hint="Wrong CAPTCHA answer.")
+            set_session_hint(
+                request, success=False, hint=gettext("Wrong CAPTCHA answer.")
+            )
             return redirect("withdraw-confirm", withdraw_id=withdraw_id)
         op.confirmation_done = True
         op.save()
@@ -1212,9 +1271,7 @@ def confirm_withdrawal(request, withdraw_id):
             op.selected_exchange_account,
             op.selected_reserve_pub,
         )
-        set_session_hint(
-            request, success=True, hint="Withdrawal successful!"
-        )
+        set_session_hint(request, success=True, hint=gettext("Withdrawal successful!"))
         request.session["just_withdrawn"] = True
         return redirect("profile")
 
@@ -1229,7 +1286,7 @@ def confirm_withdrawal(request, withdraw_id):
             amount=op.amount.stringify(settings.TALER_DIGITS),
             exchange=op.selected_exchange_account.user,
             is_success=is_success,
-            hint=hint
+            hint=hint,
         )
         return render(request, "withdraw_confirm.html", context)
     raise Exception("not reached")
@@ -1263,9 +1320,9 @@ def wire_transfer(amount, debit_account, credit_account, subject, request_uid=No
                 return JsonResponse(
                     data=dict(
                         hint="conflicting transfer with same request_uid exists",
-                        ec=5600,
+                        ec=ErrorCode.BANK_WITHDRAWAL_OPERATION_RESERVE_SELECTION_CONFLICT,
                     ),
-                    status=409,
+                    status=HTTPStatus.CONFLICT,
                 )
 
     LOGGER.info(
@@ -1388,20 +1445,25 @@ def bank_accounts_api_get_withdrawal(request, user, acct_id, wid):
         }
     )
 
+
 def withdraw_abort_internal(wid):
     op = TalerWithdrawOperation.objects.get(withdraw_id=wid)
     if op.confirmation_done:
-        return dict(status=409, hint="can't abort confirmed withdrawal")
+        return dict(status=HTTPStauts.CONFLICT, hint="can't abort confirmed withdrawal")
     op.aborted = True
     op.save()
-    return dict(status=200, hint="withdraw successfully aborted")
+    return dict(status=HTTPStatus.OK, hint="withdraw successfully aborted")
 
 
 @require_POST
 @login_required
 def abort_withdrawal(request, withdraw_id):
     internal_status = withdraw_abort_internal(withdraw_id)
-    set_session_hint(request, success=internal_status["status"] == 200, hint=internal_status["hint"])
+    set_session_hint(
+        request,
+        success=internal_status["status"] == HTTPStatus.OK,
+        hint=internal_status["hint"],
+    )
     return redirect("profile")
 
 
@@ -1420,6 +1482,7 @@ def bank_accounts_api_abort_withdrawal(request, user, acct_id, wid):
         dict(hint=internal_status["hint"]), status=internal_status["status"]
     )
 
+
 @csrf_exempt
 @require_POST
 @login_via_headers
@@ -1432,9 +1495,11 @@ def bank_accounts_api_confirm_withdrawal(request, user, acct_id, wid):
         )
     op = TalerWithdrawOperation.objects.get(withdraw_id=wid)
     if op.confirmation_done:
-        return JsonResponse(dict(), status=200)
+        return JsonResponse(dict(), status=HTTPStatus.OK)
     if op.aborted:
-        return JsonResponse(dict(hint="can't confirm aborted withdrawal"), status=409)
+        return JsonResponse(
+            dict(hint="can't confirm aborted withdrawal"), status=HTTPStatus.CONFLICT
+        )
 
     with transaction.atomic():
         if op.selection_done:
@@ -1446,4 +1511,4 @@ def bank_accounts_api_confirm_withdrawal(request, user, acct_id, wid):
             )
         op.confirmation_done = True
         op.save()
-    return JsonResponse(dict(), status=200)
+    return JsonResponse(dict(), status=HTTPStatus.OK)
